// File: pythia8_test1.cc
//
//#@# 1: p p --> (Z,gamma* -> l+ l-) + jets cross section [mb] at LHC
//#@# 2: A fraction of events with 2 isolated leptons plus >=2 jets
//#@# 3: 1+(max. pz nonconservation, GeV). The error is fixed 
//#@# 4: 0.01+(max. px, py nonconservation, GeV). The error is fixed
//#@# 5: Average final state particles multiplicity
//#@# 6: Average final state charged particles multiplicity
//#@#
//#@# Tune is fixed to the default tune in pythia8 1XX: Tune:ee = 3 Tune:pp = 5
//#@#
//#@# This test creates LHE file Zg_pythia8.lhe
//
#include "Pythia8/Pythia.h"

using namespace Pythia8; 

//**************************************************************************

int main() {

  // Generator. Process selection.
  Pythia pythia;
  pythia.readString("Main:numberOfEvents = 1000");
  pythia.readString("WeakBosonAndParton:qqbar2gmZg = on");
  pythia.readString("WeakBosonAndParton:qg2gmZq = on");
  pythia.readString("PhaseSpace:pTHatMin = 20.");
  pythia.readString("Tune:ee = 3");
  pythia.readString("Tune:pp = 5");

  // suppress full listing of first events in pythia8 > 160
  pythia.readString("Next:numberShowEvent = 0");

  // Choose decay modes for Z0 : switch off everything but Z0 -> leptons.
  pythia.readString("23:onMode = off");
  pythia.readString("23:onIfAny = 11 13 15");

  //LHC initialization.

  pythia.readString("Beams:eCM = 14000.");
  pythia.init();

  // Shorthand for the event and for settings.
  Event& event = pythia.event;
  Settings& settings = pythia.settings;

  // Extract settings to be used in the main program.
  int nEvent = settings.mode("Main:numberOfEvents"); // default is 1000
  int nList = settings.mode("Next:numberShowEvent");
  int nShow = settings.mode("Next:numberCount");
  int nAbort = settings.mode("Main:timesAllowErrors");
  //bool showChangedSettings = settings.flag("Init:showChangedSettings");
  //bool showAllSettings = settings.flag("Init:showAllSettings");
  //bool showAllParticleData = settings.flag("Init:showAllParticleData");

  // List data and particle data. Commented out because by default are listed at the init step
  //if (showChangedSettings) settings.listChanged();
  //if (showAllSettings) settings.listAll();
  //if (showAllParticleData) pythia.particleData.listAll();

  // Begin event loop.
  int nShowPace = max(1,nShow);
  int iAbort = 0; 
  double pdevmaxz=0.;
  double pdevmaxxy=0.;
  double nFinAverage = 0.;
  double nChAverage = 0.;
  double nFinSqAverage = 0.;
  double nChSqAverage = 0.;
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {
    if (iEvent%nShowPace == 0) cout << " Now begin event " 
      << iEvent << endl;

    // Generate events. Quit if too many failures.
    if (!pythia.next()) {
      if (++iAbort < nAbort) continue;
      cout << " Event generation aborted prematurely, owing to error!\n"; 
      exit(1);
    }
 
    // List first few events, both hard process and complete events.
    if (iEvent < nList) { 
//      pythia.process.list();
//      event.list();
    }

    // Reset quantities to be summed over event.
    int nfin = 0;
    int nch = 0;
    Vec4 pSum = - (event[1].p() + event[2].p());

    // Loop over particles in the event.
    for (int i = 0; i < event.size(); ++i) {
      // Specialize to final particles. Total multiplicity and momentum.
      if (event[i].status() > 0) {
        ++nfin;
        if (event[i].isCharged()) ++nch;
        pSum += event[i].p();
      // End of loop over (final/all) particles.
      }
    }
    if(fabs(pSum.pz()) > pdevmaxz) pdevmaxz = fabs(pSum.pz());
    if(fabs(pSum.px()) > pdevmaxxy) pdevmaxxy = fabs(pSum.px());
    if(fabs(pSum.py()) > pdevmaxxy) pdevmaxxy = fabs(pSum.py());

    nFinAverage += (double)nfin;
    nChAverage += (double)nch;
    nFinSqAverage += (double)(nfin*nfin);
    nChSqAverage += (double)(nch*nch);

  // End of event loop.
  }

  // Final statistics.
  pythia.stat();

  cout << endl;
  cout << " max nonconservation of pz = " << pdevmaxz << endl;
  cout << " max nonconservation of pxy = " << pdevmaxxy << endl;

  nFinAverage /= ((double)nEvent);
  nChAverage /= ((double)nEvent);
  nFinSqAverage /= ((double)nEvent);
  nChSqAverage /= ((double)nEvent);

  // Done.
  return 0;
}
